/* context2.c -- manages the libSieve opaque context and callbacks.
 * Aaron Stone
 * $Id: runtests.c,v 1.1 2006/02/07 03:53:12 sodabrew Exp $
 */
/* * * *
 * Copyright 2005 by Aaron Stone
 *
 * Licensed under the GNU Lesser General Public License (LGPL)
 * version 2.1, and other versions at the author's discretion.
 * * * */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#ifndef HAVE_GETOPT
# include "getopt.h"
# include "getopt.c"
#else
# include <unistd.h>
#endif

#include "sieve2.h"
#include "sieve2_error.h"

int main(int argc, char *argv[])
{
	int validate_only = 0;
	int c, usage_error = 0;
	int res, exitcode = 0;
	char *message, *script;

	while ((c = getopt(argc, argv, "v")) != EOF)
	switch (c) {
	case 'v':
		validate_only = 1;
		break;
	default:
		usage_error = 1;
		break;
	}

	if (validate_only) {
		if ((argc - optind < 1)) {
			usage_error = 1;
		} else {
			script = argv[optind];
		}
	} else {
		if ((argc - optind) < 2) {
			usage_error = 1;
		} else {
			script = argv[optind];
			message = argv[optind+1];
		}
	}

	if (usage_error) {
		fprintf(stderr, "usage:\n");
		fprintf(stderr, "%s script message\n", argv[0]);
		fprintf(stderr, "%s -v script\n", argv[0]);
		exitcode = 1;
		goto endnofree;
	}

	for (i = 0; i < MAX_TESTS; i++) {
	}


	exitcode |= my_context->error_parse;
	exitcode |= my_context->error_runtime;

freesieve:
	res = sieve2_free(&sieve2_context);
	if (res != SIEVE2_OK) {
		printf("Error %d when calling sieve2_free: %s\n",
			res, sieve2_errstr(res));
		exitcode = 1;
	}

	if (my_context->m_buf) free(my_context->m_buf);
	if (my_context->s_buf) free(my_context->s_buf);

	if (my_context) free(my_context);

endnofree:
	return exitcode;
}

/* END OF THE EXAMPLE */

